#!/usr/bin/python

#
# Copyright (c) 2011, Carnegie Mellon University.
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
# 1. Redistributions of source code must retain the above copyright
#    notice, this list of conditions and the following disclaimer.
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions and the following disclaimer in the
#    documentation and/or other materials provided with the distribution.
# 3. Neither the name of the University nor the names of its contributors
#    may be used to endorse or promote products derived from this software
#    without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
# A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT
# HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
# INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
# BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
# OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
# AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
# LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY
# WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.
#

from libcloud.types import Provider, NodeState
from libcloud.providers import get_driver
from newvcloud import *
from userpass import *
import pickle

import sys

CACHE_NAME = "namecache.pkl"

def debug(st):
    print "DEBUG: %s" % st

def get_vcd_conn(username, password):
    Driver = VCloudNodeDriver 
    conn = Driver(username, password)
    conn.connection.host = VCD_ENDPT
    return conn

#dump a new namecache to file
#newcache: new contents of namecache
def putcache(newcache):
    fp = open(CACHE_NAME,"wb")
    pickle.dump(newcache, fp)
    fp.close()

#load cache from file
def getcache():
    try:
        fp = open(CACHE_NAME,"rb")
    except IOError:
        putcache({})
        fp = open(CACHE_NAME,"rb")
    cache = pickle.load(fp)
    fp.close()
    return cache

def usage(thisn):
    print "usage: %s [vmname] [user] [pass]" % thisn
    print "  vmname: the name of the VM to remote to"
    print "  user: the vCloud username (xxx@pdl)"
    print "  pass: the vCloud user's password"

def main(argv):
    argv = sys.argv
    if len(argv) != 4:
        usage(argv[0])
        sys.exit(1)
    
    i_vmname = argv[1]
    i_user = argv[2]
    i_pass = argv[3]
    
    cache = getcache()
    conn = get_vcd_conn(i_user, i_pass)
    #check cache
    if i_vmname in cache:
        debug("Found %s in cache" % i_vmname)
    else:
        debug("Cache miss for %s" % i_vmname)
        children = {}
        vapps = conn.list_nodes()
        for app in vapps:
            for vm in app.children:
                children[vm.name] = vm.id
        debug("Writing new cache")
        putcache(children)
        cache = children
    
    #die if not found.
    if not i_vmname in cache:
        debug("Could not find VM %s" % i_vmname)
        sys.exit(1)
        
    vmid = cache[i_vmname]
    debug("API endpt = %s" % vmid)
    tkt = conn.get_screenticket_from_id(vmid)
    
    if tkt == None:
        debug("Could not get screen ticket!")
        sys.exit(1)
    
    conn.exec_vmrc(tkt, VMRC_LOC)
    
    debug("Done.")

if __name__ == '__main__':
    main(sys.argv)
    